#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2024, Arcangelo Massari <arcangelo.massari@unibo.it>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.

import unittest
from unittest.mock import MagicMock, patch

from oc_ocdm.counter_handler.redis_counter_handler import RedisCounterHandler


class TestRedisCounterHandler(unittest.TestCase):

    def setUp(self):
        self.mock_redis = MagicMock()
        with patch('redis.Redis', return_value=self.mock_redis):
            self.counter_handler = RedisCounterHandler(host='localhost', port=6379, db=0)

    def test_set_counter(self):
        with self.subTest("Set counter for bibliographic resource"):
            self.counter_handler.set_counter(1, "br", supplier_prefix="060")
            self.mock_redis.set.assert_called_with("br:060", 1)

        with self.subTest("Set counter for identifier"):
            self.counter_handler.set_counter(5, "id", supplier_prefix="060")
            self.mock_redis.set.assert_called_with("id:060", 5)

        with self.subTest("Set counter for agent role"):
            self.counter_handler.set_counter(10, "ar", supplier_prefix="060")
            self.mock_redis.set.assert_called_with("ar:060", 10)

        with self.subTest("Set counter for responsible agent"):
            self.counter_handler.set_counter(15, "ra", supplier_prefix="060")
            self.mock_redis.set.assert_called_with("ra:060", 15)

        with self.subTest("Set counter for resource embodiment"):
            self.counter_handler.set_counter(20, "re", supplier_prefix="060")
            self.mock_redis.set.assert_called_with("re:060", 20)

        with self.subTest("Set provenance counter"):
            self.counter_handler.set_counter(2, "br", "se", "1", "060")
            self.mock_redis.set.assert_called_with("br:060:1:se", 2)

        with self.subTest("Wrong inputs"):
            with self.assertRaises(ValueError):
                self.counter_handler.set_counter(-1, "br", supplier_prefix="060")

    def test_read_counter(self):
        with self.subTest("Read counter for bibliographic resource"):
            self.mock_redis.get.return_value = "1"
            result = self.counter_handler.read_counter("br", supplier_prefix="060")
            self.assertEqual(result, 1)
            self.mock_redis.get.assert_called_with("br:060")

        with self.subTest("Read provenance counter"):
            self.mock_redis.get.return_value = "2"
            result = self.counter_handler.read_counter("br", "se", "1", "060")
            self.assertEqual(result, 2)
            self.mock_redis.get.assert_called_with("br:060:1:se")

        with self.subTest("Read non-existent counter"):
            self.mock_redis.get.return_value = None
            result = self.counter_handler.read_counter("br", supplier_prefix="060")
            self.assertEqual(result, 0)

    def test_increment_counter(self):
        with self.subTest("Increment counter for bibliographic resource"):
            self.mock_redis.incr.return_value = 2
            result = self.counter_handler.increment_counter("br", supplier_prefix="060")
            self.assertEqual(result, 2)
            self.mock_redis.incr.assert_called_with("br:060")

        with self.subTest("Increment provenance counter"):
            self.mock_redis.incr.return_value = 3
            result = self.counter_handler.increment_counter("br", "se", "1", "060")
            self.assertEqual(result, 3)
            self.mock_redis.incr.assert_called_with("br:060:1:se")

    def test_set_metadata_counter(self):
        with self.subTest("Set metadata counter"):
            self.counter_handler.set_metadata_counter(5, "di", "http://dataset/")
            self.mock_redis.set.assert_called_with("metadata:http://dataset/:di", 5)

        with self.subTest("Wrong inputs"):
            with self.assertRaises(ValueError):
                self.counter_handler.set_metadata_counter(-1, "di", "http://dataset/")
            with self.assertRaises(ValueError):
                self.counter_handler.set_metadata_counter(1, "di", None)

    def test_read_metadata_counter(self):
        with self.subTest("Read metadata counter"):
            self.mock_redis.get.return_value = "5"
            result = self.counter_handler.read_metadata_counter("di", "http://dataset/")
            self.assertEqual(result, 5)
            self.mock_redis.get.assert_called_with("metadata:http://dataset/:di")

        with self.subTest("Read non-existent metadata counter"):
            self.mock_redis.get.return_value = None
            result = self.counter_handler.read_metadata_counter("di", "http://dataset/")
            self.assertEqual(result, 0)

        with self.subTest("Wrong inputs"):
            with self.assertRaises(ValueError):
                self.counter_handler.read_metadata_counter("di", None)

    def test_increment_metadata_counter(self):
        with self.subTest("Increment metadata counter"):
            self.mock_redis.incr.return_value = 6
            result = self.counter_handler.increment_metadata_counter("di", "http://dataset/")
            self.assertEqual(result, 6)
            self.mock_redis.incr.assert_called_with("metadata:http://dataset/:di")

        with self.subTest("Wrong inputs"):
            with self.assertRaises(ValueError):
                self.counter_handler.increment_metadata_counter("di", None)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import os
import unittest

from rdflib import Namespace, URIRef

from oc_ocdm.graph.graph_set import GraphSet
from oc_ocdm.prov.prov_set import ProvSet
from oc_ocdm.support.support import (find_paths,
                                     get_ordered_contributors_from_br)


class TestSupport(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'
    pro = Namespace("http://purl.org/spar/pro/")

    def setUp(self):
        self.graph_set = GraphSet("http://test/", "./info_dir/", "", False)
        self.prov_set = ProvSet(self.graph_set, "http://test/", "./info_dir/", False)

        self.br = self.graph_set.add_br(self.resp_agent)

    def _prepare_ordered_authors_list(self, list_len):
        # First of all, we must cleanup the GraphSet:
        self.br.remove_contributor()

        for ar in self.graph_set.get_ar():
            del self.graph_set.res_to_entity[ar.res]
        for ra in self.graph_set.get_ra():
            del self.graph_set.res_to_entity[ra.res]

        # Then, we initialize a new well-formed linked list of authors:
        ar_ordered_list = []

        for i in range(list_len):
            ra = self.graph_set.add_ra(self.resp_agent)

            ar = self.graph_set.add_ar(self.resp_agent)
            ar.create_author()
            ar.is_held_by(ra)

            self.br.has_contributor(ar)
            ar_ordered_list.append(ar)

        # Here each node of the list gets linked to the next one:
        for i in range(list_len - 1):
            ar_ordered_list[i].has_next(ar_ordered_list[i + 1])

        return ar_ordered_list

    @staticmethod
    def _extract_ra_list(ar_list):
        # Here the RA list is built and returned:
        ra_list = []
        for i in range(len(ar_list)):
            ra = ar_list[i].get_is_held_by()
            if ra is not None:
                ra_list.append(ra)

        return ra_list

    def test_get_ordered_contributors_from_br(self):
        list_len = 100
        with self.subTest("Empty linked list"):
            result = get_ordered_contributors_from_br(self.br, self.pro.author)

            self.assertIsNotNone(result)
            self.assertListEqual([], result)

        with self.subTest("Well-formed linked list"):
            correct_list = self._prepare_ordered_authors_list(list_len)
            result = get_ordered_contributors_from_br(self.br, self.pro.author)

            self.assertIsNotNone(result)
            ar_list = self._extract_ra_list(correct_list)
            self.assertListEqual(ar_list, result)

        with self.subTest("Linked list with a loop"):
            correct_list = self._prepare_ordered_authors_list(list_len)
            # Here we corrupt the well-formed linked list introducing a loop:
            correct_list[80].has_next(correct_list[50])

            self.assertRaises(ValueError, get_ordered_contributors_from_br, self.br, self.pro.author)

        with self.subTest("Linked list split in two sub-lists"):
            correct_list = self._prepare_ordered_authors_list(list_len)
            # Here we corrupt the well-formed linked list introducing a loop:
            correct_list[64].remove_next()

            self.assertRaises(ValueError, get_ordered_contributors_from_br, self.br, self.pro.author)
    
    def test_find_paths(self):
        cur_dir_path, cur_file_path = find_paths(
            res = URIRef('https://w3id.org/oc/meta/br/060169'),
            base_dir = os.path.join('support', 'test', 'data', 'rdf'),
            base_iri = 'https://w3id.org/oc/meta',
            default_dir = '_',
            dir_split = 10000,
            n_file_item = 1000,
            is_json = True)
        self.assertEqual((cur_dir_path, cur_file_path), (os.path.join('support', 'test', 'data', 'rdfbr', '060', '10000'), os.path.join('support', 'test', 'data', 'rdfbr', '060', '10000', '1000.json')))

    def test_find_paths_prov(self):
        cur_dir_path, cur_file_path = find_paths(
            res = URIRef('https://w3id.org/oc/meta/br/060165/prov/se/1'),
            base_dir = os.path.join('support', 'test', 'data', 'rdf'),
            base_iri = 'https://w3id.org/oc/meta',
            default_dir = '_',
            dir_split = 10000,
            n_file_item = 1000,
            is_json = True)
        self.assertEqual((cur_dir_path, cur_file_path), (os.path.join('support', 'test', 'data', 'rdfbr', '060', '10000', '1000', 'prov'), os.path.join('support', 'test', 'data', 'rdfbr', '060', '10000', '1000', 'prov', 'se.json')))


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

import re
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from typing import Optional
    from rdflib import URIRef

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.decorators import accepts_only
from oc_ocdm.graph.entities.bibliographic_entity import BibliographicEntity


class ResourceEmbodiment(BibliographicEntity):
    """Resource embodiment (short: re): the particular physical or digital format in which a
       bibliographic resource was made available by its publisher."""

    @accepts_only('re')
    def merge(self, other: ResourceEmbodiment) -> None:
        """
        The merge operation allows combining two ``ResourceEmbodiment`` entities into a single one,
        by marking the second entity as to be deleted while also copying its data into the current
        ``ResourceEmbodiment``. Moreover, every triple from the containing ``GraphSet`` referring to the second
        entity gets "redirected" to the current entity: **every other reference contained inside a
        different source (e.g. a triplestore) must be manually handled by the user!**

        In case of functional properties, values from the current entity get overwritten
        by those coming from the second entity while, in all other cases, values from the
        second entity are simply appended to those of the current entity. In this context,
        ``rdfs:label`` is considered as a functional property, while ``rdf:type`` is not.

        :param other: The entity which will be marked as to be deleted and whose properties will
         be merged into the current entity.
        :type other: ResourceEmbodiment
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        super(ResourceEmbodiment, self).merge(other)

        media_type: Optional[URIRef] = other.get_media_type()
        if media_type is not None:
            self.has_media_type(media_type)

        starting_page: Optional[str] = other.get_starting_page()
        if starting_page is not None:
            self.has_starting_page(starting_page)

        ending_page: Optional[str] = other.get_ending_page()
        if ending_page is not None:
            self.has_ending_page(ending_page)

        url: Optional[URIRef] = other.get_url()
        if url is not None:
            self.has_url(url)

    # HAS FORMAT
    def get_media_type(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``dcterms:format`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_format)
        return uri

    @accepts_only('thing')
    def has_media_type(self, thing_res: URIRef) -> None:
        """
        Setter method corresponding to the ``dcterms:format`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `It allows one to specify the IANA media type of the embodiment.`

        :param thing_res: The value that will be set as the object of the property related to this method
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_media_type()
        self.g.add((self.res, GraphEntity.iri_has_format, thing_res))

    def remove_media_type(self) -> None:
        """
        Remover method corresponding to the ``dcterms:format`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, GraphEntity.iri_has_format, None))

    # HAS FIRST PAGE
    def get_starting_page(self) -> Optional[str]:
        """
        Getter method corresponding to the ``prism:startingPage`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(GraphEntity.iri_starting_page)

    @accepts_only('literal')
    def has_starting_page(self, string: str) -> None:
        """
        Setter method corresponding to the ``prism:startingPage`` RDF predicate.

        The string gets internally preprocessed by eventually removing dashes and everything
        that follows them (e.g. '22-45' becomes '22').

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The first page of the bibliographic resource according to the current embodiment.`

        :param string: The value that will be set as the object of the property related to this method. **It must
          be a string that starts with an integer number.**
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_starting_page()
        if re.search("[-–]+", string) is None:
            page_number = string
        else:
            page_number = re.sub("[-–]+.*$", "", string)
        self._create_literal(GraphEntity.iri_starting_page, page_number)

    def remove_starting_page(self) -> None:
        """
        Remover method corresponding to the ``prism:startingPage`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, GraphEntity.iri_starting_page, None))

    # HAS LAST PAGE
    def get_ending_page(self) -> Optional[str]:
        """
        Getter method corresponding to the ``prism:endingPage`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(GraphEntity.iri_ending_page)

    @accepts_only('literal')
    def has_ending_page(self, string: str) -> None:
        """
        Setter method corresponding to the ``prism:endingPage`` RDF predicate.

        The string gets internally preprocessed by eventually removing dashes and everything
        that comes before them (e.g. '22-45' becomes '45').

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The last page of the bibliographic resource according to the current embodiment.`

        :param string: The value that will be set as the object of the property related to this method. **It must
          be a string that ends with an integer number.**
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_ending_page()
        if re.search("[-–]+", string) is None:
            page_number = string
        else:
            page_number = re.sub("^.*[-–]+", "", string)
        self._create_literal(GraphEntity.iri_ending_page, page_number)

    def remove_ending_page(self) -> None:
        """
        Remover method corresponding to the ``prism:endingPage`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, GraphEntity.iri_ending_page, None))

    # HAS URL
    def get_url(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``frbr:exemplar`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(GraphEntity.iri_has_url)
        return uri

    @accepts_only('thing')
    def has_url(self, thing_res: URIRef) -> None:
        """
        Setter method corresponding to the ``frbr:exemplar`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The URL at which the embodiment of the bibliographic resource is available.`

        :param thing_res: The value that will be set as the object of the property related to this method
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_url()
        self.g.add((self.res, GraphEntity.iri_has_url, thing_res))

    def remove_url(self) -> None:
        """
        Remover method corresponding to the ``frbr:exemplar`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, GraphEntity.iri_has_url, None))

    # HAS TYPE
    def create_digital_embodiment(self) -> None:
        """
        Setter method corresponding to the ``rdf:type`` RDF predicate.
        It implicitly sets the object value ``fabio:DigitalManifestation``.

        **WARNING: the OCDM specification admits at most two types for an entity.
        The main type cannot be edited or removed. Any existing secondary type
        will be overwritten!**

        `It identifies the particular type of the embodiment, either digital or print.`

        :return: None
        """
        self._create_type(GraphEntity.iri_digital_manifestation)

    def create_print_embodiment(self) -> None:
        """
        Setter method corresponding to the ``rdf:type`` RDF predicate.
        It implicitly sets the object value ``fabio:PrintObject``.

        **WARNING: the OCDM specification admits at most two types for an entity.
        The main type cannot be edited or removed. Any existing secondary type
        will be overwritten!**

        `It identifies the particular type of the embodiment, either digital or print.`

        :return: None
        """
        self._create_type(GraphEntity.iri_print_object)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from rdflib import XSD

from oc_ocdm.decorators import accepts_only
from oc_ocdm.metadata.metadata_entity import MetadataEntity

if TYPE_CHECKING:
    from typing import Optional
    from rdflib import URIRef


class Distribution(MetadataEntity):
    """Distribution (short: di): an accessible form of a dataset, for example a downloadable
       file."""

    @accepts_only('di')
    def merge(self, other: Distribution) -> None:
        """
        The merge operation allows combining two ``Distribution`` entities into a single one,
        by marking the second entity as to be deleted while also copying its data into the current
        ``Distribution``. Moreover, every triple from the containing ``MetadataSet`` referring to the second
        entity gets "redirected" to the current entity: **every other reference contained inside a
        different source (e.g. a triplestore) must be manually handled by the user!**

        In case of functional properties, values from the current entity get overwritten
        by those coming from the second entity while, in all other cases, values from the
        second entity are simply appended to those of the current entity. In this context,
        ``rdfs:label`` is considered as a functional property, while ``rdf:type`` is not.

        :param other: The entity which will be marked as to be deleted and whose properties will
         be merged into the current entity.
        :type other: Distribution
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        super(Distribution, self).merge(other)

        title: Optional[str] = other.get_title()
        if title is not None:
            self.has_title(title)

        description: Optional[str] = other.get_description()
        if description is not None:
            self.has_description(description)

        pub_date: Optional[str] = other.get_publication_date()
        if pub_date is not None:
            self.has_publication_date(pub_date)

        byte_size: Optional[str] = other.get_byte_size()
        if byte_size is not None:
            self.has_byte_size(byte_size)

        license_uri: Optional[URIRef] = other.get_license()
        if license_uri is not None:
            self.has_license(license_uri)

        download_url: Optional[URIRef] = other.get_download_url()
        if download_url is not None:
            self.has_download_url(download_url)

        media_type: Optional[URIRef] = other.get_media_type()
        if media_type is not None:
            self.has_media_type(media_type)

    # HAS TITLE
    def get_title(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcterms:title`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_title)

    @accepts_only('literal')
    def has_title(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcterms:title`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The title of the distribution.`

        :param string: The value that will be set as the object of the property related to this method
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_title()
        self._create_literal(MetadataEntity.iri_title, string)

    def remove_title(self) -> None:
        """
        Remover method corresponding to the ``dcterms:title`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_title, None))

    # HAS DESCRIPTION
    def get_description(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcterms:description`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_description)

    @accepts_only('literal')
    def has_description(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcterms:description`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `A short textual description of the content of the distribution.`

        :param string: The value that will be set as the object of the property related to this method
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_description()
        self._create_literal(MetadataEntity.iri_description, string)

    def remove_description(self) -> None:
        """
        Remover method corresponding to the ``dcterms:description`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_description, None))

    # HAS PUBLICATION DATE
    def get_publication_date(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcterms:issued`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_issued)

    @accepts_only('literal')
    def has_publication_date(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcterms:issued`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The date of first publication of the distribution.`

        :param string: The value that will be set as the object of the property related to this method. **It must
          be a string compliant with the** ``xsd:dateTime`` **datatype.**
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_publication_date()
        self._create_literal(MetadataEntity.iri_issued, string, XSD.dateTime, False)

    def remove_publication_date(self) -> None:
        """
        Remover method corresponding to the ``dcterms:issued`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_issued, None))

    # HAS BYTE SIZE
    def get_byte_size(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcat:byte_size`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_byte_size)

    @accepts_only('literal')
    def has_byte_size(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcat:byte_size`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The size in bytes of the distribution.`

        :param string: The value that will be set as the object of the property related to this method. **It must
          be a string compliant with the** ``xsd:decimal`` **datatype.**
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_byte_size()
        self._create_literal(MetadataEntity.iri_byte_size, string, XSD.decimal)

    def remove_byte_size(self) -> None:
        """
        Remover method corresponding to the ``dcat:byte_size`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_byte_size, None))

    # HAS LICENSE
    def get_license(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``dcterms:license`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_license)

    @accepts_only('thing')
    def has_license(self, thing_res: URIRef) -> None:
        """
        Setter method corresponding to the ``dcterms:license`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The resource describing the license associated with the data in the distribution.`

        :param thing_res: The value that will be set as the object of the property related to this method
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_license()
        self.g.add((self.res, MetadataEntity.iri_license, thing_res))

    def remove_license(self) -> None:
        """
        Remover method corresponding to the ``dcterms:license`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_license, None))

    # HAS DOWNLOAD URL
    def get_download_url(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``dcat:downloadURL`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_download_url)

    @accepts_only('thing')
    def has_download_url(self, thing_res: URIRef) -> None:
        """
        Setter method corresponding to the ``dcat:downloadURL`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The URL of the document where the distribution is stored.`

        :param thing_res: The value that will be set as the object of the property related to this method
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_download_url()
        self.g.add((self.res, MetadataEntity.iri_download_url, thing_res))

    def remove_download_url(self) -> None:
        """
        Remover method corresponding to the ``dcat:downloadURL`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_download_url, None))

    # HAS_MEDIA_TYPE
    def get_media_type(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``dcat:mediaType`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(MetadataEntity.iri_media_type)

    @accepts_only('thing')
    def has_media_type(self, thing_res: URIRef) -> None:
        """
        Setter method corresponding to the ``dcat:mediaType`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The file type of the representation of the distribution (according to IANA media types).`

        :param thing_res: The value that will be set as the object of the property related to this method
        :type thing_res: URIRef
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_media_type()
        self.g.add((self.res, MetadataEntity.iri_media_type, thing_res))

    def remove_media_type(self) -> None:
        """
        Remover method corresponding to the ``dcat:mediaType`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, MetadataEntity.iri_media_type, None))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from rdflib import URIRef, Literal, RDF

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet


class TestResourceEmbodiment(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def setUp(self):
        self.re = self.graph_set.add_re(self.resp_agent)

    def test_has_media_type(self):
        media_type = URIRef("http://test/MediaType")
        result = self.re.has_media_type(media_type)
        self.assertIsNone(result)

        triple = self.re.res, GraphEntity.iri_has_format, media_type
        self.assertIn(triple, self.re.g)

    def test_has_starting_page(self):
        starting_page = "15"
        result = self.re.has_starting_page(starting_page)
        self.assertIsNone(result)

        triple = self.re.res, GraphEntity.iri_starting_page, Literal(starting_page)
        self.assertIn(triple, self.re.g)

    def test_has_ending_page(self):
        ending_page = "288"
        result = self.re.has_ending_page(ending_page)
        self.assertIsNone(result)

        triple = self.re.res, GraphEntity.iri_ending_page, Literal(ending_page)
        self.assertIn(triple, self.re.g)

    def test_has_url(self):
        url = URIRef("http://test/url")
        result = self.re.has_url(url)
        self.assertIsNone(result)

        triple = self.re.res, GraphEntity.iri_has_url, url
        self.assertIn(triple, self.re.g)

    def test_create_digital_embodiment(self):
        result = self.re.create_digital_embodiment()
        self.assertIsNone(result)

        triple = self.re.res, RDF.type, GraphEntity.iri_digital_manifestation
        self.assertIn(triple, self.re.g)

    def test_create_print_embodiment(self):
        result = self.re.create_print_embodiment()
        self.assertIsNone(result)

        triple = self.re.res, RDF.type, GraphEntity.iri_print_object
        self.assertIn(triple, self.re.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
import unittest

from rdflib import URIRef, XSD, Literal, RDF

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet

from oc_ocdm.counter_handler.sqlite_counter_handler import SqliteCounterHandler


class TestCitation(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("https://w3id.org/oc/index/coci/", "./info_dir/", "", False)

    def setUp(self):
        self.br1 = self.graph_set.add_br(self.resp_agent)
        self.br2 = self.graph_set.add_br(self.resp_agent)
        self.ci = self.graph_set.add_ci(self.resp_agent)
        self.ci_oci = self.graph_set.add_ci(self.resp_agent, res=URIRef('https://w3id.org/oc/index/coci/ci/020010000023601000907630001040258020000010008010559090238044008040338381018136312231227010309014203370037122439026325-020010305093619112227370109090937010437073701020309'))

    def test_has_citing_entity(self):
        result = self.ci.has_citing_entity(self.br1)
        self.assertIsNone(result)

        triple = self.ci.res, GraphEntity.iri_has_citing_entity, self.br1.res
        self.assertIn(triple, self.ci.g)

    def test_create_cited_entity(self):
        result = self.ci.has_cited_entity(self.br2)
        self.assertIsNone(result)

        triple = self.ci.res, GraphEntity.iri_has_cited_entity, self.br2.res
        self.assertIn(triple, self.ci.g)

    def test_has_citation_creation_date(self):
        with self.subTest("date is '2020-05-25'"):
            string = "2020-05-25"
            datatype = XSD.date
            result = self.ci.has_citation_creation_date(string)
            self.assertIsNone(result)

            triple = self.ci.res, GraphEntity.iri_has_citation_creation_date, Literal(string, datatype=datatype,
                                                                                      normalize=False)
            self.assertIn(triple, self.ci.g)
        with self.subTest("date is '2020-05'"):
            string = "2020-05"
            datatype = XSD.gYearMonth
            result = self.ci.has_citation_creation_date(string)
            self.assertIsNone(result)

            triple = self.ci.res, GraphEntity.iri_has_citation_creation_date, Literal(string, datatype=datatype,
                                                                                      normalize=False)
            self.assertIn(triple, self.ci.g)
        with self.subTest("date is '2020'"):
            string = "2020"
            datatype = XSD.gYear
            result = self.ci.has_citation_creation_date(string)
            self.assertIsNone(result)

            triple = self.ci.res, GraphEntity.iri_has_citation_creation_date, Literal(string, datatype=datatype,
                                                                                      normalize=False)
            self.assertIn(triple, self.ci.g)

    def test_has_citation_time_span(self):
        duration = "P2Y6M5DT12H35M30S"  # 2 years, 6 months, 5 days, 12 hours, 35 minutes, 30 seconds
        datatype = XSD.duration
        result = self.ci.has_citation_time_span(duration)
        self.assertIsNone(result)

        triple = self.ci.res, GraphEntity.iri_has_citation_time_span, Literal(duration, datatype=datatype,
                                                                              normalize=False)
        self.assertIn(triple, self.ci.g)

    def test_has_citation_characterization(self):
        characterization = URIRef("http://test/characterization")
        result = self.ci.has_citation_characterization(characterization)
        self.assertIsNone(result)

        triple = self.ci.res, GraphEntity.iri_citation_characterisation, characterization
        self.assertIn(triple, self.ci.g)

    def test_create_self_citation(self):
        result = self.ci.create_self_citation()
        self.assertIsNone(result)

        triple = self.ci.res, RDF.type, GraphEntity.iri_self_citation
        self.assertIn(triple, self.ci.g)

    def test_create_affiliation_self_citation(self):
        result = self.ci.create_affiliation_self_citation()
        self.assertIsNone(result)

        triple = self.ci.res, RDF.type, GraphEntity.iri_affiliation_self_citation
        self.assertIn(triple, self.ci.g)

    def test_create_author_network_self_citation(self):
        result = self.ci.create_author_network_self_citation()
        self.assertIsNone(result)

        triple = self.ci.res, RDF.type, GraphEntity.iri_author_network_self_citation
        self.assertIn(triple, self.ci.g)

    def test_create_author_self_citation(self):
        result = self.ci.create_author_self_citation()
        self.assertIsNone(result)

        triple = self.ci.res, RDF.type, GraphEntity.iri_author_self_citation
        self.assertIn(triple, self.ci.g)

    def test_create_funder_self_citation(self):
        result = self.ci.create_funder_self_citation()
        self.assertIsNone(result)

        triple = self.ci.res, RDF.type, GraphEntity.iri_funder_self_citation
        self.assertIn(triple, self.ci.g)

    def test_create_journal_self_citation(self):
        result = self.ci.create_journal_self_citation()
        self.assertIsNone(result)

        triple = self.ci.res, RDF.type, GraphEntity.iri_journal_self_citation
        self.assertIn(triple, self.ci.g)

    def test_create_journal_cartel_citation(self):
        result = self.ci.create_journal_cartel_citation()
        self.assertIsNone(result)

        triple = self.ci.res, RDF.type, GraphEntity.iri_journal_cartel_citation
        self.assertIn(triple, self.ci.g)

    def test_create_distant_citation(self):
        result = self.ci.create_distant_citation()
        self.assertIsNone(result)
        triple = self.ci.res, RDF.type, GraphEntity.iri_distant_citation
        self.assertIn(triple, self.ci.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

import json
import os
import time
from datetime import datetime
from typing import TYPE_CHECKING
from zipfile import ZIP_DEFLATED, ZipFile

from rdflib import ConjunctiveGraph, URIRef
from SPARQLWrapper import SPARQLWrapper

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.metadata.metadata_entity import MetadataEntity
from oc_ocdm.prov.prov_entity import ProvEntity
from oc_ocdm.reader import Reader
from oc_ocdm.support.query_utils import get_update_query
from oc_ocdm.support.reporter import Reporter
from oc_ocdm.support.support import find_paths
from filelock import FileLock

if TYPE_CHECKING:
    from typing import Any, Dict, List, Optional, Set, Tuple

    from oc_ocdm.abstract_entity import AbstractEntity
    from oc_ocdm.abstract_set import AbstractSet


class Storer(object):

    def __init__(self, abstract_set: AbstractSet, repok: Reporter = None, reperr: Reporter = None,
                 context_map: Dict[str, Any] = None, default_dir: str = "_", dir_split: int = 0,
                 n_file_item: int = 1, output_format: str = "json-ld", zip_output: bool = False, modified_entities: set = None) -> None:
        # We only accept format strings that:
        # 1. are supported by rdflib
        # 2. correspond to an output format which is effectively either NT or NQ
        # The only exception to this rule is the 'json-ld' format, which is the default value of 'output_format'.
        supported_formats: Set[str] = {'application/n-triples', 'ntriples', 'nt', 'nt11',
                                       'application/n-quads', 'nquads', 'json-ld'}
        if output_format not in supported_formats:
            raise ValueError(f"Given output_format '{self.output_format}' is not supported."
                             f" Available formats: {supported_formats}.")
        else:
            self.output_format: str = output_format
        self.zip_output = zip_output
        self.dir_split: int = dir_split
        self.n_file_item: int = n_file_item
        self.default_dir: str = default_dir if default_dir != "" else "_"
        self.a_set: AbstractSet = abstract_set
        self.modified_entities = modified_entities

        if context_map is not None:
            self.context_map: Dict[str, Any] = context_map
        else:
            self.context_map: Dict[str, Any] = {}

        if self.output_format == "json-ld":
            for context_url in self.context_map:
                ctx_file_path: Any = self.context_map[context_url]
                if type(ctx_file_path) == str and os.path.isfile(ctx_file_path):
                    # This expensive operation is done only when it's really needed
                    with open(ctx_file_path, 'rt', encoding='utf-8') as ctx_f:
                        self.context_map[context_url] = json.load(ctx_f)

        if repok is None:
            self.repok: Reporter = Reporter(prefix="[Storer: INFO] ")
        else:
            self.repok: Reporter = repok

        if reperr is None:
            self.reperr: Reporter = Reporter(prefix="[Storer: ERROR] ")
        else:
            self.reperr: Reporter = reperr

    def store_graphs_in_file(self, file_path: str, context_path: str = None) -> None:
        self.repok.new_article()
        self.reperr.new_article()
        self.repok.add_sentence("Store the graphs into a file: starting process")

        cg: ConjunctiveGraph = ConjunctiveGraph()
        for g in self.a_set.graphs():
            cg.addN([item + (g.identifier,) for item in list(g)])

        self._store_in_file(cg, file_path, context_path)

    def _store_in_file(self, cur_g: ConjunctiveGraph, cur_file_path: str, context_path: str = None) -> None:
        # Note: the following lines from here and until 'cur_json_ld' are a sort of hack for including all
        # the triples of the input graph into the final stored file. Somehow, some of them are not written
        # in such file otherwise - in particular the provenance ones.
        new_g: ConjunctiveGraph = ConjunctiveGraph()
        for s, p, o in cur_g.triples((None, None, None)):
            g_iri: Optional[URIRef] = None
            for g_context in cur_g.contexts((s, p, o)):
                g_iri = g_context.identifier
                break
            new_g.addN([(s, p, o, g_iri)])

        zip_file_path = cur_file_path.replace(os.path.splitext(cur_file_path)[1], ".zip")
        
        if self.zip_output:
            with ZipFile(zip_file_path, mode="w", compression=ZIP_DEFLATED, allowZip64=True) as zip_file:
                self._write_graph(new_g, zip_file, cur_file_path, context_path)
        else:
            # Handle non-zipped output directly to a file
            self._write_graph(new_g, None, cur_file_path, context_path)

        self.repok.add_sentence(f"File '{cur_file_path}' added.")

    def _write_graph(self, graph: ConjunctiveGraph, zip_file: ZipFile, cur_file_path, context_path):
        if self.output_format == "json-ld":
            # Serialize the graph in JSON-LD format
            cur_json_ld = json.loads(graph.serialize(format="json-ld", context=self.context_map.get(context_path)))
            if context_path is not None and context_path in self.context_map:
                if isinstance(cur_json_ld, dict):
                    cur_json_ld["@context"] = context_path
                else:  # When cur_json_ld is a list
                    for item in cur_json_ld:
                        item["@context"] = context_path

            # Determine how to write based on zip file presence
            if zip_file is not None:
                dumped_json = json.dumps(cur_json_ld, ensure_ascii=False).encode('utf-8')
                zip_file.writestr(zinfo_or_arcname=os.path.basename(cur_file_path), data=dumped_json)
            else:
                with open(cur_file_path, 'wt', encoding='utf-8') as f:
                    json.dump(cur_json_ld, f, ensure_ascii=False)
        else:
            # Handle other RDF formats
            if zip_file is not None:
                rdf_serialization = graph.serialize(destination=None, format=self.output_format, encoding="utf-8")
                zip_file.writestr(zinfo_or_arcname=os.path.basename(cur_file_path), data=rdf_serialization)
            else:
                graph.serialize(destination=cur_file_path, format=self.output_format, encoding="utf-8")

    def store_all(self, base_dir: str, base_iri: str, context_path: str = None, process_id: int|str = None) -> List[str]:
        self.repok.new_article()
        self.reperr.new_article()

        self.repok.add_sentence("Starting the process")

        relevant_paths: Dict[str, list] = dict()
        for entity in self.a_set.res_to_entity.values():
            is_relevant = True
            if self.modified_entities is not None and URIRef(entity.res.split('/prov/se/')[0]) not in self.modified_entities:
                is_relevant = False
            if is_relevant:
                cur_dir_path, cur_file_path = self._dir_and_file_paths(entity.res, base_dir, base_iri, process_id)
                if not os.path.exists(cur_dir_path):
                    os.makedirs(cur_dir_path)
                relevant_paths.setdefault(cur_file_path, list())
                relevant_paths[cur_file_path].append(entity)

        for relevant_path, entities_in_path in relevant_paths.items():
            stored_g = None
            # Here we try to obtain a reference to the currently stored graph
            output_filepath = relevant_path.replace(os.path.splitext(relevant_path)[1], ".zip") if self.zip_output else relevant_path
            lock = FileLock(f"{output_filepath}.lock")
            with lock:
                if os.path.exists(output_filepath):
                    stored_g = Reader(context_map=self.context_map).load(output_filepath)
                if stored_g is None:
                    stored_g = ConjunctiveGraph()
                for entity_in_path in entities_in_path:
                    self.store(entity_in_path, stored_g, relevant_path, context_path, False)
                self._store_in_file(stored_g, relevant_path, context_path)

        return list(relevant_paths.keys())

    def store(self, entity: AbstractEntity, destination_g: ConjunctiveGraph, cur_file_path: str, context_path: str = None, store_now: bool = True) -> ConjunctiveGraph:
        self.repok.new_article()
        self.reperr.new_article()

        try:
            if isinstance(entity, ProvEntity):
                quads: List[Tuple] = []
                graph_identifier: URIRef = entity.g.identifier
                for triple in entity.g.triples((entity.res, None, None)):
                    quads.append((*triple, graph_identifier))
                destination_g.addN(quads)
            elif isinstance(entity, GraphEntity) or isinstance(entity, MetadataEntity):
                if entity.to_be_deleted:
                    destination_g.remove((entity.res, None, None, None))
                else:
                    if len(entity.preexisting_graph) > 0:
                        """
                        We're not in 'append mode', so we need to remove
                        the entity that we're going to overwrite.
                        """
                        destination_g.remove((entity.res, None, None, None))
                    """
                    Here we copy data from the entity into the stored graph.
                    If the entity was marked as to be deleted, then we're
                    done because we already removed all of its triples.
                    """
                    quads: List[Tuple] = []
                    graph_identifier: URIRef = entity.g.identifier
                    for triple in entity.g.triples((entity.res, None, None)):
                        quads.append((*triple, graph_identifier))
                    destination_g.addN(quads)

            if store_now:
                self._store_in_file(destination_g, cur_file_path, context_path)

            return destination_g
        except Exception as e:
            self.reperr.add_sentence(f"[1] It was impossible to store the RDF statements in {cur_file_path}. {e}")

    def upload_and_store(self, base_dir: str, triplestore_url: str, base_iri: str, context_path: str = None,
                         batch_size: int = 10) -> None:
        stored_graph_path: List[str] = self.store_all(base_dir, base_iri, context_path)

        # If some graphs were not stored properly, then no one will be uploaded to the triplestore
        # Anyway, we should highlight those ones that could have been added in principle, by
        # mentioning them with a ".notuploaded" marker
        if None in stored_graph_path:
            for file_path in stored_graph_path:
                if file_path is not None:
                    # Create a marker for the file not uploaded in the triplestore
                    open(f'{file_path}.notuploaded', 'wt', encoding='utf-8').close()
                    self.reperr.add_sentence("[2] "
                                             f"The statements contained in the JSON-LD file '{file_path}' "
                                             "were not uploaded into the triplestore.")
        else:  # All the files have been stored
            self.upload_all(triplestore_url, base_dir, batch_size)

    def _dir_and_file_paths(self, res: URIRef, base_dir: str, base_iri: str, process_id: int|str = None) -> Tuple[str, str]:
        is_json: bool = (self.output_format == "json-ld")
        return find_paths(res, base_dir, base_iri, self.default_dir, self.dir_split, self.n_file_item, is_json=is_json, process_id=process_id)

    @staticmethod
    def _class_to_entity_type(entity: AbstractEntity) -> Optional[str]:
        if isinstance(entity, GraphEntity):
            return "graph"
        elif isinstance(entity, ProvEntity):
            return "prov"
        elif isinstance(entity, MetadataEntity):
            return "metadata"
        else:
            return None

    def upload_all(self, triplestore_url: str, base_dir: str = None, batch_size: int = 10, save_queries: bool = False) -> bool:
        self.repok.new_article()
        self.reperr.new_article()

        if batch_size <= 0:
            batch_size = 10

        query_string: str = ""
        added_statements: int = 0
        removed_statements: int = 0
        skipped_queries: int = 0
        result: bool = True

        if save_queries:
            to_be_uploaded_dir = os.path.join(base_dir, "to_be_uploaded")
            os.makedirs(to_be_uploaded_dir, exist_ok=True)

        for idx, entity in enumerate(self.a_set.res_to_entity.values()):
            update_query, n_added, n_removed = get_update_query(entity, entity_type=self._class_to_entity_type(entity))

            if update_query == "":
                skipped_queries += 1
            else:
                index = idx - skipped_queries
                if index == 0:
                    # First query
                    query_string = update_query
                    added_statements = n_added
                    removed_statements = n_removed
                elif index % batch_size == 0:
                    # batch_size-multiple query
                    if save_queries:
                        self._save_query(query_string, to_be_uploaded_dir, added_statements, removed_statements)
                    else:
                        result &= self._query(query_string, triplestore_url, base_dir, added_statements, removed_statements)
                    query_string = update_query
                    added_statements = n_added
                    removed_statements = n_removed
                else:
                    # Accumulated query
                    query_string += " ; " + update_query
                    added_statements += n_added
                    removed_statements += n_removed

        if query_string != "":
            if save_queries:
                self._save_query(query_string, to_be_uploaded_dir, added_statements, removed_statements)
            else:
                result &= self._query(query_string, triplestore_url, base_dir, added_statements, removed_statements)

        return result

    def _save_query(self, query_string: str, directory: str, added_statements: int, removed_statements: int) -> None:
        timestamp = datetime.now().strftime('%Y-%m-%d-%H-%M-%S-%f')
        file_name = f"{timestamp}_add{added_statements}_remove{removed_statements}.sparql"
        file_path = os.path.join(directory, file_name)
        with open(file_path, 'w', encoding='utf-8') as f:
            f.write(query_string)
            
    def upload(self, entity: AbstractEntity, triplestore_url: str, base_dir: str = None) -> bool:
        self.repok.new_article()
        self.reperr.new_article()

        update_query, n_added, n_removed = get_update_query(entity, entity_type=self._class_to_entity_type(entity))

        return self._query(update_query, triplestore_url, base_dir, n_added, n_removed)

    def execute_query(self, query_string: str, triplestore_url: str) -> bool:
        self.repok.new_article()
        self.reperr.new_article()

        return self._query(query_string, triplestore_url)

    def _query(self, query_string: str, triplestore_url: str, base_dir: str = None,
            added_statements: int = 0, removed_statements: int = 0) -> bool:
        if query_string != "":
            attempt = 0
            max_attempts = 3
            wait_time = 5  # Initial wait time in seconds

            while attempt < max_attempts:
                try:
                    sparql: SPARQLWrapper = SPARQLWrapper(triplestore_url)
                    sparql.setQuery(query_string)
                    sparql.setMethod('POST')

                    sparql.query()

                    self.repok.add_sentence(
                        f"Triplestore updated with {added_statements} added statements and "
                        f"with {removed_statements} removed statements.")

                    return True

                except Exception as e:
                    attempt += 1
                    self.reperr.add_sentence("[3] "
                                            f"Attempt {attempt} failed. Graph was not loaded into the "
                                            f"triplestore due to communication problems: {e}")
                    if attempt < max_attempts:
                        self.reperr.add_sentence(f"Retrying in {wait_time} seconds...")
                        time.sleep(wait_time)
                        wait_time *= 2  # Double the wait time for the next attempt

                    if base_dir is not None and attempt == max_attempts:
                        self.reperr.add_sentence("[3] "
                                                "Graph was not loaded into the "
                                                f"triplestore due to communication problems: {e}")
                        tp_err_dir: str = base_dir + os.sep + "tp_err"
                        if not os.path.exists(tp_err_dir):
                            os.makedirs(tp_err_dir)
                        cur_file_err: str = tp_err_dir + os.sep + \
                            datetime.now().strftime('%Y-%m-%d-%H-%M-%S-%f_not_uploaded.txt')
                        with open(cur_file_err, 'wt', encoding='utf-8') as f:
                            f.write(query_string)

        return False


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2023, Arcangelo Massari <arcangelo.massari@unibo.it>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.

import os
import unittest

from rdflib import URIRef
from SPARQLWrapper import JSON, SPARQLWrapper

from oc_ocdm.graph import GraphSet
from oc_ocdm.reader import Reader


class TestReader(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.endpoint = 'http://127.0.0.1:9999/blazegraph/sparql'
        BASE = os.path.join('oc_ocdm', 'test', 'reader')
        server = SPARQLWrapper(cls.endpoint)
        query = 'LOAD <file:' + os.path.abspath(os.path.join(BASE, f'br.nt')).replace('\\', '/') + '> INTO GRAPH <' + f'https://w3id.org/oc/meta/' + '>'
        server.setQuery(query)
        server.query()
    
    def test_import_entity_from_triplestore(self):
        reader = Reader()
        g_set = GraphSet('https://w3id.org/oc/meta')
        reader.import_entity_from_triplestore(g_set, self.endpoint, URIRef('https://w3id.org/oc/meta/br/0605'), 'https://orcid.org/0000-0002-8420-0696', False)
        self.assertEqual(set(str(s) for s in g_set.res_to_entity.keys()), {'https://w3id.org/oc/meta/br/0605'})


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2016, Silvio Peroni <essepuntato@gmail.com>
#
# Permission to use, copy, modify, and/or distribute this software for any purpose
# with or without fee is hereby granted, provided that the above copyright notice
# and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT,
# OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
# DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
from __future__ import annotations

from typing import TYPE_CHECKING

from rdflib import XSD

from oc_ocdm.decorators import accepts_only
from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.prov.prov_entity import ProvEntity

if TYPE_CHECKING:
    from typing import List, Optional

    from rdflib import URIRef


class SnapshotEntity(ProvEntity):
    """Snapshot of entity metadata (short: se): a particular snapshot recording the
    metadata associated with an individual entity (either a bibliographic entity or an
    identifier) at a particular date and time, including the agent, such as a person,
    organisation or automated process that created or modified the entity metadata."""

    # HAS CREATION DATE
    def get_generation_time(self) -> Optional[str]:
        """
        Getter method corresponding to the ``prov:generatedAtTime`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(ProvEntity.iri_generated_at_time)

    @accepts_only('literal')
    def has_generation_time(self, string: str) -> None:
        """
        Setter method corresponding to the ``prov:generatedAtTime`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The date on which a particular snapshot of a bibliographic entity's metadata was
        created.`

        :param string: The value that will be set as the object of the property related to this method. **It must
          be a string compliant with the** ``xsd:dateTime`` **datatype.**
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_generation_time()
        self._create_literal(ProvEntity.iri_generated_at_time, string, XSD.dateTime)

    def remove_generation_time(self) -> None:
        """
        Remover method corresponding to the ``prov:generatedAtTime`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, ProvEntity.iri_generated_at_time, None))

    # HAS INVALIDATION DATE
    def get_invalidation_time(self) -> Optional[str]:
        """
        Getter method corresponding to the ``prov:invalidatedAtTime`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(ProvEntity.iri_invalidated_at_time)

    @accepts_only('literal')
    def has_invalidation_time(self, string: str) -> None:
        """
        Setter method corresponding to the ``prov:invalidatedAtTime`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The date on which a snapshot of a bibliographic entity's metadata was invalidated due
        to an update (e.g. a correction, or the addition of some metadata that was not specified
        in the previous snapshot), or due to a merger of the entity with another one.`

        :param string: The value that will be set as the object of the property related to this method. **It must
          be a string compliant with the** ``xsd:dateTime`` **datatype.**
        :type string: str
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.remove_invalidation_time()
        self._create_literal(ProvEntity.iri_invalidated_at_time, string, XSD.dateTime)

    def remove_invalidation_time(self) -> None:
        """
        Remover method corresponding to the ``prov:invalidatedAtTime`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, ProvEntity.iri_invalidated_at_time, None))

    # IS SNAPSHOT OF
    def get_is_snapshot_of(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``prov:specializationOf`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(ProvEntity.iri_specialization_of)
        return uri

    def is_snapshot_of(self, en_res: GraphEntity) -> None:
        """
        Setter method corresponding to the ``prov:specializationOf`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `This property is used to link a snapshot of entity metadata to the bibliographic entity
        to which the snapshot refers.`

        :param en_res: The value that will be set as the object of the property related to this method
        :type en_res: GraphEntity
        :return: None
        """
        self.remove_is_snapshot_of()
        self.g.add((self.res, ProvEntity.iri_specialization_of, en_res.res))

    def remove_is_snapshot_of(self) -> None:
        """
        Remover method corresponding to the ``prov:specializationOf`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, ProvEntity.iri_specialization_of, None))

    # IS DERIVED FROM
    def get_derives_from(self) -> List[ProvEntity]:
        """
        Getter method corresponding to the ``prov:wasDerivedFrom`` RDF predicate.

        :return: A list containing the requested values if found, None otherwise
        """
        uri_list: List[URIRef] = self._get_multiple_uri_references(ProvEntity.iri_was_derived_from, 'se')
        result: List[ProvEntity] = []
        for uri in uri_list:
            # TODO: what is the prov_subject of these snapshots?
            result.append(self.p_set.add_se(None, uri))
        return result

    @accepts_only('se')
    def derives_from(self, se_res: ProvEntity) -> None:
        """
        Setter method corresponding to the ``prov:wasDerivedFrom`` RDF predicate.

        `This property is used to identify the immediately previous snapshot of entity metadata
        associated with the same bibliographic entity.`

        :param se_res: The value that will be set as the object of the property related to this method
        :type se_res: ProvEntity
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        self.g.add((self.res, ProvEntity.iri_was_derived_from, se_res.res))

    @accepts_only('se')
    def remove_derives_from(self, se_res: ProvEntity = None) -> None:
        """
        Remover method corresponding to the ``prov:wasDerivedFrom`` RDF predicate.

        **WARNING: this is a non-functional property, hence, if the parameter
        is None, any existing value will be removed!**

        :param se_res: If not None, the specific object value that will be removed from the property
         related to this method (defaults to None)
        :type se_res: SnapshotEntity
        :raises TypeError: if the parameter is of the wrong type
        :return: None
        """
        if se_res is not None:
            self.g.remove((self.res, ProvEntity.iri_was_derived_from, se_res.res))
        else:
            self.g.remove((self.res, ProvEntity.iri_was_derived_from, None))

    # HAS PRIMARY SOURCE
    def get_primary_source(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``prov:hadPrimarySource`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(ProvEntity.iri_had_primary_source)
        return uri

    @accepts_only('thing')
    def has_primary_source(self, any_res: URIRef) -> None:
        """
        Setter method corresponding to the ``prov:hadPrimarySource`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `This property is used to identify the primary source from which the metadata
        described in the snapshot are derived (e.g. Crossref, as the result of querying the
        CrossRef API).`

        :param any_res: The value that will be set as the object of the property related to this method
        :type any_res: URIRef
        :return: None
        """
        self.remove_primary_source()
        self.g.add((self.res, ProvEntity.iri_had_primary_source, any_res))

    def remove_primary_source(self) -> None:
        """
        Remover method corresponding to the ``prov:hadPrimarySource`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, ProvEntity.iri_had_primary_source, None))

    # HAS UPDATE ACTION
    def get_update_action(self) -> Optional[str]:
        """
        Getter method corresponding to the ``oco:hasUpdateQuery`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(ProvEntity.iri_has_update_query)

    @accepts_only('literal')
    def has_update_action(self, string: str) -> None:
        """
        Setter method corresponding to the ``oco:hasUpdateQuery`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The UPDATE SPARQL query that specifies which data, associated to the bibliographic
        entity in consideration, have been modified (e.g. for correcting a mistake) in the
        current snapshot starting from those associated to the previous snapshot of the entity.`

        :param string: The value that will be set as the object of the property related to this method
        :type string: str
        :return: None
        """
        self.remove_update_action()
        self._create_literal(ProvEntity.iri_has_update_query, string)

    def remove_update_action(self) -> None:
        """
        Remover method corresponding to the ``oco:hasUpdateQuery`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, ProvEntity.iri_has_update_query, None))

    # HAS DESCRIPTION
    def get_description(self) -> Optional[str]:
        """
        Getter method corresponding to the ``dcterms:description`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        return self._get_literal(ProvEntity.iri_description)

    @accepts_only('literal')
    def has_description(self, string: str) -> None:
        """
        Setter method corresponding to the ``dcterms:description`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `A textual description of the events that have resulted in the current snapshot (e.g. the
        creation of the initial snapshot, the creation of a new snapshot following the
        modification of the entity to which the metadata relate, or the creation of a new
        snapshot following the merger with another entity of the entity to which the previous
        snapshot related).`

        :param string: The value that will be set as the object of the property related to this method
        :type string: str
        :return: None
        """
        self.remove_description()
        self._create_literal(ProvEntity.iri_description, string)

    def remove_description(self) -> None:
        """
        Remover method corresponding to the ``dcterms:description`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, ProvEntity.iri_description, None))

    # IS ATTRIBUTED TO
    def get_resp_agent(self) -> Optional[URIRef]:
        """
        Getter method corresponding to the ``prov:wasAttributedTo`` RDF predicate.

        :return: The requested value if found, None otherwise
        """
        uri: Optional[URIRef] = self._get_uri_reference(ProvEntity.iri_was_attributed_to)
        return uri

    @accepts_only('thing')
    def has_resp_agent(self, se_agent: URIRef) -> None:
        """
        Setter method corresponding to the ``prov:wasAttributedTo`` RDF predicate.

        **WARNING: this is a functional property, hence any existing value will be overwritten!**

        `The agent responsible for the creation of the current entity snapshot.`

        :param se_agent: The value that will be set as the object of the property related to this method
        :type se_agent: URIRef
        :return: None
        """
        self.remove_resp_agent()
        self.g.add((self.res, ProvEntity.iri_was_attributed_to, se_agent))

    def remove_resp_agent(self) -> None:
        """
        Remover method corresponding to the ``prov:wasAttributedTo`` RDF predicate.

        :return: None
        """
        self.g.remove((self.res, ProvEntity.iri_was_attributed_to, None))
